c w182.f
c Hf/W system, model w. Moon giant impact (MGI).
c Reference: Yu & Jacobson (2011), PNAS
c Miroslav Broz (miroslav.broz@email.cz), Jun 7th 2019

c observational constraints:
c eps_W(CHUR) = 1.9 +- 0.2
c f^Hf/W = 15 +- 3

c observed concentrations (Tab. S1)
c CW2 = 0.01575  ! ppm
c CW3 = 0.4906  ! ppm
c CW3/CW2 = 31.15

      program w182
      include 'common.inc'

c constants
      integer NMAX
      parameter (NMAX=100001)

c input
      real*8 tstart,tstop,dtout
      character*255 filename
      integer n
      real*8 t_nbody(NMAX),m_nbody(NMAX)

      real*8 M,C2,f2,e2
      real*8 dM_dt,dC2_dt,df2_dt,de2_dt

      real*8 tau0
      real*8 hf182_hf180_CAI
      real*8 w182_w183_CAI
      real*8 hf180_w183_CHUR
      real*8 w182_w183_CHUR
      real*8 hf180_w182_CHUR
      real*8 hf182_hf180_CHUR
      real*8 lambda_hf182
      real*8 t,tout,eps
      real*8 P_CMB,T_CMB
      real*8 M_
      integer phase

c functions
      real*8 interp

c constants
      tau0 = 4567.d0
      lambda_hf182 = 0.078d0  ! +- 0.001 [My^-1]
      gamma = 0.325d0  ! core mass fraction

      hf182_hf180_CAI = 1.0d-4  ! CAIs == "solar"
      w182_w183_CAI = 1.850664  ! +- 37

      hf180_w183_CHUR = 2.836d0
      w182_w183_CHUR = 1.850948  ! +- 37

      hf180_w182_CHUR = hf180_w183_CHUR/w182_w183_CHUR
      hf182_hf180_CHUR = hf182_hf180_CAI
      q_W = hf180_w182_CHUR*1.d4

c input parameters
      read(*,*) tstart
      read(*,*) tstop
      read(*,*) dt
      read(*,*) dtout
      read(*,*) D
      read(*,*) D_MGI
      read(*,*) filename
      
c header
      write(*,*) '# q_W = ', q_W
      write(*,*) '# D = ', D
      write(*,*) '# D_MGI = ', D_MGI

c read m(t)
      n = NMAX
      call read_nbody(filename,t_nbody,m_nbody,n)
      write(*,*) '# filename = ', filename
      write(*,*) '# n = ', n

      write(*,*) '# ',
     :  't [My] &                  ',
     :  'M [M_E] &                ',
     :  'C_W of reservoir 2 (mantle, BSE) [] & ',
     :  'f^Hf/W [] &    ',
     :  'eps_W [] &               ',
     :  'D_W [] &                   ',
     :  'P_CMB [GPa] &             ',
     :  'T_CMB [K] &             '

c initial conditions
      M = interp(t_nbody,m_nbody,n,tstart)
      C2 = 0.01575d-6
      C1 = C2*(gamma*D+1.d0-gamma)
      f2 = gamma*D/(1.d0-gamma)
      e2 = 0.d0

c main cycle
      t = tstart
      tout = tstart
      eps = 1.d-8

      do while (t.le.tstop)

        QWstar = lambda_hf182*exp(-lambda_hf182*t)*hf182_hf180_CHUR*q_W

        M_ = interp(t_nbody,m_nbody,n,t)
        dM_dt = (M_-M)/dt

        if ((M_-M).gt.1.0d-3) then
          write(*,*) '# collision at t = ', t
          phase = 3
          call phase3(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
        else
          phase = 1
          call phase1(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
        endif

c output
        if ((t.ge.tout-eps).or.(phase.eq.3)) then
          write(*,*) t,M,C2,f2,e2,D,P_CMB,T_CMB
        endif
        if (t.ge.tout-eps) then
          tout = tout + dtout
        endif

c        if (t.lt.t_PE) then
c          call phase1(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
c        else if (t.lt.t_MGI) then
c          call phase2(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
c        else if (i1st.eq.0) then
c          call phase3(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
c          i1st = 1
c        else
c          call phase4(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
c        endif

        M = M + dM_dt*dt
        C2 = C2 + dC2_dt*dt
        f2 = f2 + df2_dt*dt
        e2 = e2 + de2_dt*dt

        t = t + dt
      enddo

      stop
      end

c C ... concentration of W
c f ... fractionation, f^Hf/W = (hf180_w183/hf180_w183_CHUR - 1)
c e ... W-182 excess, eps_W = (w182_w183/w182_w183_CHUR - 1) * 10^4
c gamma ... core mass fraction
c alpha ... Earth accretion rate
c D ... metal/silicate partition coefficient, DW = CW_metal/CW_silicate
 
c 1 ... nebula reservoir
c 2 ... mantle
c 3 ... core

c accretion
      subroutine phase1(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
      include 'common.inc'
      real*8 M,C2,f2,e2
      real*8 dM_dt,dC2_dt,df2_dt,de2_dt

c      dM_dt = alpha*(M_tau0-M)
      dC2_dt = (C1-(gamma*D+1.d0-gamma)*C2)/((1.d0-gamma)*M)*dM_dt
      df2_dt = ((gamma*D-C1/C2)*f2 + gamma*D)
     :  / ((1.d0-gamma)*M)*dM_dt
      de2_dt = QWstar*f2 - C1/(C2*(1.d0-gamma))*dM_dt/M*e2
      return
      end

c hiatus
      subroutine phase2(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
      include 'common.inc'
      real*8 M,C2,f2,e2
      real*8 dM_dt,dC2_dt,df2_dt,de2_dt

      dM_dt = 0.d0
      dC2_dt = 0.d0
      df2_dt = 0.d0
      de2_dt = QWstar*f2 - C1/(C2*(1.d0-gamma))*dM_dt/M*e2
      return
      end

c Moon giant impact
      subroutine phase3(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
      include 'common.inc'
      real*8 M,C2,f2,e2
      real*8 dM_dt,dC2_dt,df2_dt,de2_dt
      real*8 a,M_,C2_,f2_,e2_

      a = (gamma*D_MGI+1.d0-gamma)/(1.d0-gamma)

      M_ = M+dM_dt*dt
      C2_ = C2*(M/M_)**a
     :  + C1/(gamma*D_MGI+1.d0-gamma)*(1.d0-(M/M_)**a)
      f2_ = f2  ! dbg
c      f2_ = f2*C2/C2_*(M/M_)**a
      e2_ = e2*C2/C2_*(M/M_)**a

      dM_dt = (M_-M)/dt
      dC2_dt = (C2_-C2)/dt
      df2_dt = (f2_-f2)/dt
      de2_dt = (e2_-e2)/dt
      return
      end

c present
      subroutine phase4(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
      implicit none
      real*8 M,C2,f2,e2
      real*8 dM_dt,dC2_dt,df2_dt,de2_dt

      call phase2(M,C2,f2,e2,dM_dt,dC2_dt,df2_dt,de2_dt)
      return
      end


c read m(t)
      subroutine read_nbody(filename,t,m,n)
      implicit none
      character*(*) filename
      integer n
      real*8 t(n),m(n)
      integer i,ierr

      open(unit=10,file=filename,status='old',iostat=ierr)
      if (ierr.ne.0) then
        write(*,*) 'Error reading ', filename
        stop
      endif
      i = 0
      ierr=0
      do while ((ierr.eq.0).and.(i.lt.n))
        i = i+1
        read(10,*,iostat=ierr) t(i),m(i)
      enddo
      n = i-1
      close(10)
      return
      end

c interpolate m(t)
      real*8 function interp(x,y,n,x0)
      implicit none
      integer n
      real*8 x(n),y(n)
      real*8 x0
      integer i

      if (x0.le.x(1)) then
        interp = y(1)
      else if (x0.ge.x(n)) then
        interp = y(n)
      else
        i = 2
        do while ((x(i).lt.x0).and.(i.lt.n))
          i = i+1
        enddo
        interp = y(i-1)
      endif
c      write(*,*) 'x0 = ', x0
c      write(*,*) 'x(1) = ', x(1)
c      write(*,*) 'x(', n, ') = ', x(n)
c      write(*,*) 'i = ', i
c      write(*,*) 'n = ', n
      return
      end


